;nyquist plug-in
;version 1
;type generate
;name "Bloops (mono)..."
;action "Generating bloops..."
;info "by David R. Sky\nReleased under terms of GNU Public License"

;control a "Attack time" int "ms" 2 1 1000 
;control d "Decay time" int "ms" 1 0 1000 
;control s "Sustain level" real "linear" 0.2 0.0 1.0
;control s-dur "Sustain duration" int "ms" 70 0 1000
;control r "Release time" int "ms" 500 1 4000  
;control type "Envelope type" int "0=inverted 1=normal" 1 0 1
;control lo-note "Lowest MIDI note #" int "MIDI#" 20 16 123
;control hi-note "Highest MIDI note #" int "MIDI#" 90 16 123
;control q "Lowpass filter q" real "q" 8.0 0.1 30.0
;control norm "Normalize individual bloops" int "0=no 1=yes" 1 0 1
;control tempo "Tempo" int "bloops per minute" 360 1 1000
;control randomness "Add tempo randomness" int "+/- percent" 0 0 100
;control count "Number of bloops" int "bloops" 16 1 119
;control repeat "Repeat bloops pattern" int "times" 16 1 119

; Bloops Generator (mono) by David R. Sky, December 3, 2005
; Thanks to Steven Jones for shedding light on the stack overflow issue
; Released under terms of the GNU Public License
; http://www.opensource.org/licenses/gpl-license.php

; initialize variables
(setf a (* 0.001 a))
(setf d (* 0.001 d))
(setf s-dur (* 0.001 s-dur))
(setf r (* 0.001 r))
(setf dur (+ a d s-dur r)) ; bloop duration
; to avoid annoying clicks in inverted envelope, 
; set pwl offset for 2ms fade-in and fade-out times
(setf offset (if (= type 0) 0.002 0.0))
(setf time (/ 60.0 tempo))
(setf count (min count 119))
(setf repeat (min repeat 119))
; if count+repeat>120, make repeat 120-count
; this prevents stack overflow issues
(setf repeat (if (> (+ count repeat) 120)
(- 120 count) repeat))
(setf lo (min lo-note hi-note))
(setf hi (max lo-note hi-note))
(setf range (+ 1 (- hi lo)))

(defun random-step ()
(+ lo (random range)))

; random note generator
(defun get-note (lo range)
(step-to-hz (random-step)))

; set pwl l (level) and s (sustain) depending on envelope type 
; l is the lowest level (0.0) in an a d s r envelope
; l is the highest point (1.0) in an inverted a d s r envelope
(setf l (float (- 1 type)))
(setf s (if (= type 1) s (- 1.0 s)))

(let (bloop-seq bleep x signal)

; all values used are global numbers so lambda list unnecessary
(defun bloop () 
(mult 
(pwl 0.0 l a (- 1.0 l) (+ a d) s (+ a d s-dur) s (+ a d s-dur r) l)
(pwl 0 0 offset 1.0 (- dur offset) 1.0 dur 0)
(lowpass2 (noise dur) (get-note lo hi) q)))

; individual normalized bloops
(defun norm-bloop ()
(setq bleep (bloop))
(setf x (peak bleep ny:all))
(scale (/ 0.95 x) bleep))

; normalize function
(defun normalize (signal)
(setf x (peak signal ny:all))
(scale (/ 0.95 x) signal))

; add tempo randomness
(defun random-time (i)
(+ (* i time)
(* 0.01 time (- (random (* (+ randomness 1) 2)) randomness))))

; sequence of 'count' mono bloops
(cond
((= norm 0) ; non-normalized individual bloops
(setq bloop-seq
(simrep (i count)
(at-abs (random-time i) (bloop)))))

((= norm 1) ; individual normalized bloops
(setq bloop-seq
(simrep (i count)
(at-abs (random-time i) (norm-bloop))))))

; normalize bloop sequence before repeating
(setq bloop-seq (normalize bloop-seq))

; 'repeat' number of bloop-seq sequences
(simrep (i repeat)
(at-abs (* i time count) (cue bloop-seq))))

